package org.wffm.gassan.model.io;

/*
 * GEML1_0to1_1.java - Short Description
 * Copyright (C) 2000 WFFM
 * koutei@erl.ee.musashi-tech.ac.jp
 * http://www.wffm.org/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.MalformedURLException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import org.wffm.gassan.model.scenario.GElement;
import org.wffm.gassan.model.scenario.GElementList;
import org.wffm.gassan.model.scenario.GElementSerializer;

/**
  * GEML Version1.01.1֕ϊNX
  *
  * @version 1.0.0, 2000/07/17
  * @author Yosuke Sato
  */
public class GEML1_0to1_1 extends GEMLConverter {
	//
	//@W
	//
	//=========================================================================
	private Document _doc = null;
	
	//
	//@
	//
	//=========================================================================
	public final static String documentClassName = "org.apache.xerces.dom.DocumentImpl";
/**
  * RXgN^
  * @param	_GEMLInputStream	GEMLInputStream
  */
public GEML1_0to1_1(GEMLInputStream _GEMLInputStream) {
	super(_GEMLInputStream);
	
	try{
		//DocumentIuWFNg̐
		_doc = (Document)Class.forName( documentClassName ).newInstance();
	}catch(Exception e){
		System.err.println("Can not create Document.");
		_doc = null;
	}
}
private Element createCreator(GElement creater){
	Element ret = null;
	
	if(creater.hasChildElements()){
		ret = _doc.createElement("creator");

		GElementList list1 = creater.getChildElements();
		GElement item1;
		for( list1.first(),item1 = list1.item() ; item1 != null ; list1.next(),item1 = list1.item() ){
			if( item1.getName().equals("createrName") ){
				ret.setAttribute("name", item1.getValue() );
			}
			else if( item1.getName().equals("createrURL") ){
				ret.setAttribute("URL", item1.getValue() );
			}
			else if( item1.getName().equals("createrE-mail") ){
				ret.setAttribute("E-mail", item1.getValue() );
			}
			else if( item1.getName().equals("createdDate") ){
				ret.setAttribute("date", item1.getValue() );
			}
		}

	}
	
	return ret;
}
private Element createGEML(GElement geml){
	Element ret = null;

	ret = _doc.createElement("geml");
	ret.setAttribute("version", "1.1" );
	
	//GEML1.0 -> GEML1.1ϊR[h
	if(geml.hasChildElements()){
		GElementList list1 = geml.getChildElements();
		GElement item1;
		
		for( list1.first(),item1 = list1.item() ; item1 != null ; list1.next(),item1 = list1.item() ){
			if( item1.getName().equals("scenario") ){
				Element scenario = createScenario(item1);
				ret.appendChild(scenario);
			}
			else if( item1.getName().equals("view") ){
				Element view = createView(item1);
				ret.appendChild(view);
			}
		}
	}
			
	return ret;
}
private Element createScenario(GElement scenario){
	Element ret = null;
	
	if(scenario.hasChildElements()){
		ret = _doc.createElement("scenario");

		String name = scenario.getAttributeValue("name");
		ret.setAttribute("name",name);
		String gsmlFile = scenario.getAttributeValue("startFile");
		ret.setAttribute("gsmlFile",gsmlFile);
		
		GElementList list1 = scenario.getChildElements();
		GElement item1;
		for( list1.first(),item1 = list1.item() ; item1 != null ; list1.next(),item1 = list1.item() ){
			if( item1.getName().equals("title") ){
				Element element = _doc.createElement("title");
				element.appendChild(_doc.createTextNode( item1.getValue() ));
				ret.appendChild(element);
			}
			else if( item1.getName().equals("creater") ){
				Element element = createCreator(item1);
				ret.appendChild(element);
			}
		}

	}
	
	return ret;
}
private Element createView(GElement view){
	Element ret = null;

	ret = _doc.createElement("view");

	String width = view.getAttributeValue("width");
	ret.setAttribute("width", width );
	String height = view.getAttributeValue("height");
	ret.setAttribute("height", height );
	String iconFile = view.getAttributeValue("icon");
	ret.setAttribute("iconFile", iconFile );
	String imageFile = view.getAttributeValue("startImage");
	ret.setAttribute("imageFile", imageFile);
	String audioFile = view.getAttributeValue("startBGM");
	ret.setAttribute("audioFile", audioFile);

	Element element;

	element = _doc.createElement("opening");
	String oldOpeningName = view.getAttributeValue("opening");

	//pbP[Wϊ
	String newOpeningName = GSML1_0to1_1.replacePackage(oldOpeningName);

	element.setAttribute("name",newOpeningName);
	ret.appendChild(element);
	
	element = _doc.createElement("ending");
	String oldEndingName = view.getAttributeValue("ending");

	//pbP[Wϊ
	String newEndingName = GSML1_0to1_1.replacePackage(oldEndingName);
	
	element.setAttribute("name",newEndingName);
	ret.appendChild(element);
		
	return ret;
}
/**
  * eXgp\bh
  */
public static void main(String[] argv){
	try{
		File file = new File(argv[0]);
		GElement element = GElement.create(file.toURL());
		
		GEMLThrowInputStream gtis = new GEMLThrowInputStream( element );
		GEMLConverter converter = new GEML1_0to1_1( gtis );
		GElement convert = converter.read();
		
		//Wo͂֏o
		//OutputFormat format=new OutputFormat(convert.getCurrentDocument());
		//format.setDoctype(null,"geml.dtd");
		//format.setEncoding("Shift_JIS");
		//GElementSerializer.serialize(System.out,format,convert);
		
		GElementSerializer.serialize(System.out,convert,"Shift_JIS");
	}
	catch( ArrayIndexOutOfBoundsException e){
		System.err.println("Usage: java GEML1_0to1_1 filename");
	}
	catch( IOException e ){
		System.err.println(argv[0] + "not Found");
	}
	catch( SAXException e){
		System.err.println("SAXError");
		System.err.println(e);
	}
	catch( GEMLIOException e){
		System.err.println("GEMLIOError");
	}
}
/**
  * GEMLǂݍ
  * @return	ϊꂽGEML
  */
public GElement read() throws GEMLIOException{
	GElement ret = null;
	
	ret = super.read();
	
	Element element = null;
	
	try{
		//System.out.println("GEML1.0 -> GEML1.1֕ϊ");
		
		//͑GEML1.0
		if(ret.getAttributeValue("version").equals("1.0")){
			element = createGEML(ret);
		}
		else{
			throw new GEMLIOException("version error.");
		}
		
		//Ԓl:GEML1.0 -> GEML1.1֕ϊꂽGEML
		
		//Document֒ǉ
		_doc.appendChild(element);
		ret = new GElement(element,ret.getBaseURL(),_doc);
	}
	catch(Exception e){
		throw new GEMLIOException("stream error.");
	}
	return ret;
}
}
