/* Public Domain Sources */

package org.wffm.gassan.view.loader.bmp;

import java.io.IOException;


/**
  * BitmapInfoHeader
  * @version 1.0.0, 2000/12/01
  * @author D.Miki
  */
public class BitmapInfoHeader extends Object {

  public static final int MONO_COLOR=2;
  public static final int VGA_COLOR=16;
  public static final int FIX_COLOR=256;
  public static final int FULL_COLOR=0;

  public static final int MONO_BIT=1;
  public static final int VGA_BIT=4;
  public static final int FIX_BIT=8;
  public static final int FULL_BIT=24;

  public static final int BI_RGB = 0;
  public static final int BI_RLE8 = 1;
  public static final int BI_RLE4 = 2;
  public static final int BI_BITFIELDS = 3;

  public static BitmapInfoHeader load(WordableDataInputStream is) throws IOException {
	int  biSize = is.readDWORD();
	int  biWidth = is.readDWORD();
	int  biHeight = is.readDWORD();
	int  biPlanes = is.readWORD();
	int  biBitCount = is.readWORD();
	int  biCompression = is.readDWORD();
	int  biSizeImage = is.readDWORD();
	int  biXPelsPerMeter = is.readDWORD();
	int  biYPelsPerMeter = is.readDWORD();
	int  biClrUsed = is.readDWORD();
	long biClrImportant = is.readDWORD();
	return new BitmapInfoHeader(
	  biSize,biWidth,biHeight,biPlanes,biBitCount,biCompression,biSizeImage,
	  biXPelsPerMeter,biYPelsPerMeter,biClrUsed,biClrImportant
	);
  }          

  private int  biSize;             // BITMAPINFOHEADER̃TCY(DWORD)
  private int  biWidth;            // isNZPʁj(DWORD)
  private int  biHeight;           // isNZPʁj(DWORD)
  private int  biPlanes;           // ɂP(WORD)
  private int  biBitCount;         // PsNZ̃J[rbg̐B1,4,8,24̂ꂩ̐(WORD)
  private int  biCompression;      // BI_RGB, BI_RLE8, BI_RLE4̂ꂩ(DWORD)
  private int  biSizeImage;        // C[W̑SoCg(DWORD)
  private int  biXPelsPerMeter;    // 0܂͐𑜓x(DWORD)
  private int  biYPelsPerMeter;    // 0܂͐𑜓x(DWORD)
  private int  biClrUsed;          // ʏ0, biBitCountȉ̃J[ɐݒ\(DWORD)
  private long biClrImportant;     // ʏ0(DWORD)

  public BitmapInfoHeader(
	int  biSize,
	int  biWidth,
	int  biHeight,
	int  biPlanes,
	int  biBitCount,
	int  biCompression,
	int  biSizeImage,
	int  biXPelsPerMeter,
	int  biYPelsPerMeter,
	int  biClrUsed,
	long biClrImportant
  ){
	super();
	this.biSize=biSize;
	this.biWidth=biWidth;
	this.biHeight=biHeight;
	this.biPlanes=biPlanes;
	this.biBitCount=biBitCount;
	this.biCompression=biCompression;
	this.biSizeImage=biSizeImage;
	this.biXPelsPerMeter=biXPelsPerMeter;
	this.biYPelsPerMeter=biYPelsPerMeter;
	this.biClrUsed=biClrUsed;
	this.biClrImportant=biClrImportant;
  }          

  public int getWidth(){return biWidth;}          
  public int getHeight(){return biHeight;}          
  public boolean isFullColor(){return (getColorCount()==BitmapInfoHeader.FULL_COLOR);}          
  public int getColorCount(){
	if(biClrUsed!=0){
	  return biClrUsed;
	}else{
	  switch(this.biBitCount){
	  case BitmapInfoHeader.MONO_BIT:
		return BitmapInfoHeader.MONO_COLOR;
	  case BitmapInfoHeader.VGA_BIT:
		return BitmapInfoHeader.VGA_COLOR;
	  case BitmapInfoHeader.FIX_BIT:
		return BitmapInfoHeader.FIX_COLOR;
	  case BitmapInfoHeader.FULL_BIT:
		return BitmapInfoHeader.FULL_COLOR;
	  default:
	  }
	}
	throw new UnsupportedOperationException("BitmapInfoHeader#getColorCount(): biBitCount is Bad Information.");
  }          
  public boolean isRLE4Compression(){return (this.biCompression==BitmapInfoHeader.BI_RLE4);}          
  public boolean isRLE8Compression(){return (this.biCompression==BitmapInfoHeader.BI_RLE8);}          

}