package org.wffm.gsmlviewer.pane;

/*
 * ImageCreaterFactory.java - Short Description
 * Copyright (C) 2000-2001 WFFM
 * koutei@erl.ee.musashi-tech.ac.jp
 * http://www.wffm.org/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
import java.awt.Component;
import java.awt.Point;
import java.awt.Dimension;
import java.net.URL;
import java.awt.Image;
import java.util.Hashtable;
import java.util.Vector;

import org.wffm.gassan.view.image.ImageCreater;
import org.wffm.gassan.view.image.ImageCreaterException;
import org.wffm.gassan.view.loader.ImageLoader;
import org.wffm.gassan.view.filter.Filter;
import org.wffm.gassan.view.composer.Composer;
import org.wffm.gassan.model.scenario.GElement;
import org.wffm.gassan.model.scenario.GElementList;

import org.wffm.gsmlviewer.ResourceFactory;

/**
  * ImageCreater̃NXTuNX𐶐NX
  * @version 1.0.1, 2001/01/20
  * @author Yosuke Sato
  */
public class ImageCreaterFactory {
/**
  * Composer𐶐
  * @param	composer	GElement
  * @param	component	Component
  * @return	ꂽComposer
  */
public static ImageCreater createComposer(GElement composer,Component comp) throws ImageCreaterException{
	Composer ret = null;

	String name = composer.getAttributeValue("name");

	GElementList list = composer.getChildElements();
	list.first();

	try{
		ret = Composer.create(name);
		
		Hashtable param = new Hashtable();
		Vector source = new Vector();
		for(GElement element = list.item(); element != null; list.next(),element = list.item()){
			if(element.getName().equals("param")){
				param.put(element.getAttributeValue("name"),element.getAttributeValue("value"));
			}
			else if(element.getName().equals("image")){
				source.add(createImageLoader(element));
			}
			else if(element.getName().equals("filter")){
				source.add(createFilter(element));
			}
		}
		ret.setComponent(comp);
		ret.setParam(param);
		ret.setSource(source);
	}
	catch(Exception e){
		throw new ImageCreaterException();
	}
	
	return ret;
}
/**
  * Filter𐶐
  * @param	filter	GElement
  * @return	ꂽFilter
  */
public static ImageCreater createFilter(GElement filter) throws ImageCreaterException{
	Filter ret = null;

	String name = filter.getAttributeValue("name");

	GElementList list = filter.getChildElements();
	list.first();

	try{
		ret = Filter.create(name);
		
		Hashtable param = new Hashtable();
		ImageCreater source = null;
		for(GElement element = list.item(); element != null; list.next(),element = list.item()){
			if(element.getName().equals("param")){
				param.put(element.getAttributeValue("name"),element.getAttributeValue("value"));
			}
			else if(element.getName().equals("image")){
				source = createImageLoader(element);
			}
		}
		ret.setParam(param);
		ret.setSource(source);
	}
	catch(Exception e){
		throw new ImageCreaterException();
	}
	
	return ret;
}
/**
  * ImageCreater𐶐
  * @param	image	GElement
  * @return	ꂽImageCreater
  */
public static ImageCreater createImageLoader(GElement image) throws ImageCreaterException{
	ImageLoader ret = null;

	URL baseURL = image.getBaseURL();
	String fileName = image.getAttributeValue("file");
	String loaderName = image.getAttributeValue("loader");
		
	try{
		String uriString = fileName;
		uriString = baseURL.toString() + fileName;
		
		URL resourceURL = new URL(uriString);
		
		if(loaderName != null){
			ret = ImageLoader.create(loaderName);
			ret.setSrc(resourceURL);
			}
		else{
			ret = ImageLoader.create("org.wffm.gassan.view.loader.DefaultImageLoader");
			ret.setSrc(resourceURL);
		}
	}catch(Exception e){
		throw new ImageCreaterException();
	}

	return ret;
}
/**
  * C[W̕`W擾
  * @param	element	GElement
  * @param	size	Dimension
  * @return	`W
  */
public static Point getImagePoint(GElement element,Dimension size){
	//element  image  compose
	Point ret = null;

	String position = element.getAttributeValue("position");
	
	if(position != null){
		if(position.equals("left"))			ret = new Point(0,0);
		else if(position.equals("center"))	ret = new Point( (int)(size.getWidth()/3) , 0);
		else if(position.equals("right"))	ret = new Point( (int)((size.getWidth()/3)*2) , 0);
	}
	if(ret == null){
		String sx = element.getAttributeValue("x");
		String sy = element.getAttributeValue("y");
		int x = 0;
		int y = 0;
		
		try{
			x = Integer.parseInt(sx);
		}catch(Exception e){
			x = 0;
		}

		try{
			y = Integer.parseInt(sy);
		}catch(Exception e){
			y = 0;
		}
		
		ret = new Point(x,y);
			
	}

	return ret;
}
}
