package org.wffm.gsmlviewer.pane;

/*
 * ImagePane.java - Short Description
 * Copyright (C) 1999-2001 WFFM
 * koutei@erl.ee.musashi-tech.ac.jp
 * http://www.wffm.org/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.awt.Point;
import java.awt.Image;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferedImage;

import javax.swing.JComponent;

import org.wffm.gassan.model.scenario.GElement;
import org.wffm.gassan.model.scenario.GElementList;
import org.wffm.gassan.view.image.ImageCreater;
import org.wffm.gassan.view.image.ImageCreaterException;
import org.wffm.gassan.view.effect.ExchangeEffect;

import org.wffm.gsmlviewer.GSMLViewer;
import org.wffm.gsmlviewer.GEMLUtility;
import org.wffm.gsmlviewer.filter.ImagePaneFilter;

/**
  * C[WNX
  * @version 1.1.5, 2001/01/18
  * @author Yosuke Sato
  */
public class ImagePane extends JComponent implements Runnable {
	
	//
	//@֘A
	//
	//=========================================================================
	private GSMLViewer _GSMLViewer = null;

	//
	//@W
	//
	//=========================================================================
	private ImagePaneFilter _filter = null;
	
	//
	//@
	//
	//=========================================================================
	private Image drawImage = null;					//`悷C[W
	private Dimension defaultSize = null;			//{TCY
	private boolean state_effect = true;			//GtFNg GtFNgs\true,ȊOfalse
	
	private int state_run = -1;						//s
	public final static int IMAGE_CHANGE_MODE = 0;
	public final static int FILTER_IN_MODE = 1;
	public final static int FILTER_OUT_MODE = 2;
	
	//
	//  [Jϐ
	//
	//=========================================================================
	private GElement imagelist = null;
	private Thread kick;
	public static final int filterTime = 500;
/**
 * \bh̋Lq}ĂB
 * 쐬 : (01/01/21 16:48:08)
 */
public ImagePane() {
	super();
}
protected BufferedImage createBufferedImage(Image image){
	BufferedImage ret = null;
	if(image != null){
		ret = new BufferedImage(image.getWidth(this),image.getHeight(this),BufferedImage.TYPE_INT_RGB);
		Graphics g = ret.createGraphics();
		g.drawImage(image,0,0,this);
	}
	else{
		ret = new BufferedImage((int)getDefaultSize().getWidth(),(int)getDefaultSize().getHeight(),BufferedImage.TYPE_INT_RGB);
	}
	return ret;
}
private BufferedImage createCombinationImage(GElement imagelist){
	BufferedImage ret = new BufferedImage((int)getDefaultSize().getWidth(),(int)getDefaultSize().getHeight(),BufferedImage.TYPE_INT_RGB);

	if(imagelist != null){
		Graphics g = ret.createGraphics();
		
		GElementList list = imagelist.getChildElements();
		list.first();
		
		for(GElement element = list.item(); element != null; list.next(),element = list.item()){
			ImageCreater creater = null;

			try{
				if(element.getName().equals("image")){
					creater = ImageCreaterFactory.createImageLoader(element);
				}
				else if(element.getName().equals("composer")){
					creater = ImageCreaterFactory.createComposer(element,this);
				}
				else if(element.getName().equals("filter")){
					creater = ImageCreaterFactory.createFilter(element);
				}
			}catch(ImageCreaterException e){
				System.err.println("Can not operate : " + element.getName() );
				creater = null;
			}
			
			if(creater != null){
				try{
					Image image = creater.getImage();
					Point point = ImageCreaterFactory.getImagePoint(element,getDefaultSize());
					g.drawImage(image,(int)point.getX(),(int)point.getY(),this);
				}catch(ImageCreaterException e){
					System.err.println("Can not create image.");
				}
			}
		}
	}

	return ret;
}
protected ExchangeEffect createEffect(GElement effect){
	ExchangeEffect ret = null;

	if(effect != null){
		if( effect.getAttributeValue("name") != null ){
			ret = ExchangeEffect.create( effect.getAttributeValue("name") );
			if(ret != null){
				ret.setParam( GEMLUtility.getParam(effect) );
			}
		}
	}
	
	return ret;
}
/**
  * tB^[C
  */
public void filterIn(){
	if(_filter != null){
		if(!_filter.getState()){
			state_run = FILTER_IN_MODE;
			start();
			
			//̃XbhI܂őҋ@
			if(kick != null){
				try{
					kick.join();
				}catch(InterruptedException e){}
			}
		
			repaint();
			timer(filterTime);
		}
	}
}
/**
  * tB^[AEg
  */
public void filterOut(){
	if(_filter != null){
		if(_filter.getState()){
			state_run = FILTER_OUT_MODE;
			start();
			
			//̃XbhI܂őҋ@
			if(kick != null){
				try{
					kick.join();
				}catch(InterruptedException e){}
			}
			
			repaint();
			timer(filterTime);
		}
	}
}
/**
  * ftHg̃TCY擾
  * @return	ftHgTCY
  */
public Dimension getDefaultSize(){
	return defaultSize;
}
/**
  * `悷C[W擾
  * @return `悷C[W
  */
public Image getDrawImage(){
	return drawImage;
}
public ImagePaneFilter getFilter(){
	return _filter;
}
/**
  * GSMLViewer擾
  * @return	GSMLVIewer
  */
public GSMLViewer getGSMLViewer(){
	return _GSMLViewer;
}
protected GElement getImageList(){
	return imagelist;
}
/**
  * GtFNgԂ擾
  * GtFNgsԂ̏ꍇtrue,ȊOfalse
  * @return	GtFNg
  */
public boolean getStateEffect(){
	return state_effect;
}
/**
  * 
  */
public void init(Dimension size){
	this.defaultSize = size;
	super.setSize(defaultSize);
	setDoubleBuffered(true);
}
public void paintComponent(Graphics g) {
	super.paintComponent(g);
	if(getDrawImage() != null)	g.drawImage(getDrawImage(),0,0,this);
}
public void run() {
	switch(state_run){
		case IMAGE_CHANGE_MODE:
			run_image();

			//Iʒm
			if(getGSMLViewer() != null){
				getGSMLViewer().reaction();
			}
			
			break;
		case FILTER_IN_MODE:
			_filter.filterIN();
			break;
		case FILTER_OUT_MODE:
			_filter.filterOUT();
			break;
		default :
			break;
	}
}
protected void run_image(){

	ExchangeEffect exchangeEffect = null;

	if(getStateEffect()){
		GElementList list = imagelist.getChildElements();
		list.first();
		list.search("effect");
		GElement effect = list.item();
		exchangeEffect = createEffect(effect);
	}

	BufferedImage afterImage = createCombinationImage(imagelist);

	try{
		exchangeEffect.setImageSize(defaultSize);
		exchangeEffect.setComponent(this);
		BufferedImage beforeImage = createBufferedImage(getDrawImage());
		exchangeEffect.setBeforeImage(beforeImage);
		exchangeEffect.setAfterImage(afterImage);
		exchangeEffect.init();
		exchangeEffect.run();
		Graphics g = getGraphics();
		g.drawImage(afterImage,0,0,this);
		setDrawImage(afterImage);
	}catch(Exception e){
		//GtFNgłȂꍇ܂͐lʏoff̏
		setDrawImage(afterImage);
	}
	/*
	if( exchangeEffect != null){
		exchangeEffect.setImageSize(defaultSize);
		exchangeEffect.setComponent(this);
		BufferedImage beforeImage = createBufferedImage(getDrawImage());
		exchangeEffect.setBeforeImage(beforeImage);
		exchangeEffect.setAfterImage(afterImage);
		exchangeEffect.init();
		exchangeEffect.run();
		Graphics g = getGraphics();
		g.drawImage(afterImage,0,0,this);
		setDrawImage(afterImage);
	}
	else{
		//GtFNgłȂꍇ܂͐lʏoff̏
		setDrawImage(afterImage);
	}
	*/
}
/**
  * `悷C[Wݒ
  * @param drawImage `悷C[W
  */
public void setDrawImage(Image drawImage){
	this.drawImage = drawImage;
	repaint();
}
public void setFilter(ImagePaneFilter _filter){
	this._filter = _filter;
}
/**
  * GSMLViewerݒ
  * @param	_GSMLViewer	GSMLViewer
  */
public void setGSMLViewer(GSMLViewer _GSMLViewer){
	this._GSMLViewer = _GSMLViewer;
}
/**
  * C[WXgݒ
  * @param	imageList	C[WXg
  */
public void setImagelist(GElement imagelist){
	this.imagelist = imagelist;

	if(imagelist != null){
		state_run = IMAGE_CHANGE_MODE;
		start();
	}
}
/**
  * GtFNgԂݒ
  * [vt̏ꍇtrue,ȊOfalse
  * @param	state_effect	GtFNg
  */
public void setStateEffect(boolean state_effect){
	this.state_effect = state_effect;
}
private void start() {
	kick = new Thread(this);
	kick.start();
}
private void stop() {
	kick = null;
}
protected void timer(int i) {
	try{
		Thread.sleep(i);
	}catch(InterruptedException e){}
}
}
